#include "OgrePathMapRenderTechnique.h"
#include "OgreTechniqueGroup.h"
#include "OgreIlluminationManager.h"


OgrePathMapRenderTechnique::OgrePathMapRenderTechnique(											
												Pass* pass,
												OgreRenderable* parentRenderable,
												OgreTechniqueGroup* parentTechniqueGroup)
							:RenderTechnique( parentRenderable, parentTechniqueGroup),
							OgreRenderTechnique(pass, parentRenderable, parentTechniqueGroup)
{		
	this->clusters = OgreIlluminationManager::getSingleton().getPathMapClusters(parentRenderable->getName());
	
	//insert new pass
	Ogre::Technique* techn = pass->getParent();
	Technique::PassIterator it = techn->getPassIterator();
	
	int index = 0;
	while(it.hasMoreElements())
	{
		if( it.getNext() == pass)
			break;
		index++;
		it.moveNext();
	}
	
	Pass* newpass = techn->createPass();
	
	newpass->setVertexProgram("GTP/PathMap_VS");
	newpass->setFragmentProgram("GTP/PathMap_PS");
	
	//bind vertex program parameters
	GpuProgramParameters* Vparams = newpass->getVertexProgramParameters().getPointer();
	Vparams->setNamedAutoConstant("WorldViewProj", GpuProgramParameters::ACT_WORLDVIEWPROJ_MATRIX);	
	//bind fragment program parameters
	GpuProgramParameters* Fparams = newpass->getFragmentProgramParameters().getPointer();
	int prmxres = clusters->pathMapResolution * clusters->count;
	int prmyres = clusters->pathMapResolution;
	while(prmxres > 4096)
	{
		prmxres /= 2;
		prmyres *= 2;		
	}
	int prmnt[2] = {prmxres / clusters->pathMapResolution, prmyres / clusters->pathMapResolution};

	float halfPixel[2] = {0.5 / prmxres, 0.5 / prmyres};
	Fparams->setNamedConstant("prmAtlasTiles",prmnt,2);
    Fparams->setNamedConstant("atlasHalfPixel", halfPixel, 2);
			
	TextureUnitState* st = newpass->createTextureUnitState();		
	st->setTextureFiltering(TFO_BILINEAR);
	st->setTextureAddressingMode(TextureUnitState::TAM_BORDER);
	st->setTextureBorderColour(ColourValue::Blue);
	st->setTextureName(clusters->pathMapTextureFilename);

	//newpass->setSceneBlending(SBT_MODULATE);
	newpass->setSceneBlending(SBF_ONE, SBF_ZERO);

	newpass->setDepthBias(1);
}

OgrePathMapRenderTechnique::~OgrePathMapRenderTechnique()
{

}

void OgrePathMapRenderTechnique::update(unsigned long frameNum)
{	
	//TODO set weights
}

OgrePathMapRenderTechniqueFactory::OgrePathMapRenderTechniqueFactory()
{
	typeName = "PathMap";
}

OgreRenderTechnique* OgrePathMapRenderTechniqueFactory::createInstance( 
										IllumTechniqueParams* params,
										Pass* pass,
										OgreRenderable* parentRenderable,
										OgreTechniqueGroup* parentTechniqueGroup)
{	
	parseParams(params);

	OgrePathMapRenderTechnique* result = new OgrePathMapRenderTechnique(
												pass,
												parentRenderable,
												parentTechniqueGroup);	

	return result;
}

